//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#include <vcl\clipbrd.hpp>
#pragma hdrstop

#include "TextWins.h"
#include "SaveDlg.h"
#include "FileMan.h"
#include "PrintDlg.h"
#include "Global.h"
#include "mixmaster.h"
#include "mainmdi.h"
#include "wiper.h"

#include <values.h>
//---------------------------------------------------------------------------
#pragma link "explbtn"
#pragma resource "*.dfm"      
TTextWin *TextWin;
//---------------------------------------------------------------------------
__fastcall TTextWin::TTextWin(TComponent* Owner)
   : TForm(Owner)
{
   Editor->Clear();
   Editor->MaxLength = 10240000;
   Editor->Font->Name = GetAppIni(EdFontName,"Courier New"); // for wrap to window
   Editor->Font->Size = GetAppIni(EdFontSize,"").ToIntDef(10);
   Editor->WordWrap = GetAppIni("WordWrap","").ToIntDef(1);
   Type = tNewNote;
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::ShowStatus(AnsiString &msg)
{
   Application->MainForm->Perform(WM_SHOWSTATUS,0,(LPARAM)msg.c_str());
}
//---------------------------------------------------------------------------
bool __fastcall TTextWin::Save()
{
   if (Type==tNewNote||Type==tNewMessage||DataKey.IsEmpty())
      return SaveAs();

   TStringList *lines = new TStringList();
   GetUnWrappedText(lines);
   PutAppData(DataKey,lines);
   delete lines;

   ShowStatus(Caption+" ... saved");
   Editor->Modified = false;
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall TTextWin::SaveAs()
{
   AnsiString path = GetSaveMessagePath(UserNotes+"only",Name());
   if (!path.IsEmpty())
   {
      DataKey = path;
      TStringList *lines = new TStringList();
      GetUnWrappedText(lines);
      PutAppData(DataKey,lines);
      delete lines;

      TextWin->SetText(GetParam(1,path,PS.c_str()),lines);
      ShowStatus(Caption+" ... saved");
      Editor->Modified = false;
      return true;
   }
   return false;
}
//---------------------------------------------------------------------------
bool __fastcall TTextWin::CheckSave()
{
   int reply;
   bool ok = true;
   if (Editor->Modified)
   {
      if (Type==tChainLog)
          reply = MsgBoxHwnd(Handle,
            "Your Chain Log is modified. Changes will be lost!\n\nSave it now?",
               "Save Log",MB_YESNOCANCEL);
      else
          reply = MsgBoxHwnd(Handle,
            "Your note is modified. Changes will be lost!\n\nSave it now?",
               "Save Note",MB_YESNOCANCEL);
      switch(reply)
      {
         case IDCANCEL: ok = false; break;
         case IDYES: Save(); break;
      }
   }
   return ok;
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::PopupMenuClick(TObject *Sender)
{
   TMenuItem *item;
   if (dynamic_cast<TPopupMenu*>(Sender)!=NULL)
   {
      bool hasSel = Editor->SelLength > 0;
      PopUndoItem->Enabled = Editor->Modified;
      PopCutItem->Enabled = hasSel;
      PopCopyItem->Enabled = hasSel;
      PopPasteItem->Enabled = Clipboard()->HasFormat(CF_TEXT);
      PopPasteAsQuoteItem->Enabled = PopPasteItem->Enabled;
      PopSelectAllItem->Enabled = Editor->Lines->Count > 0;
      PopInsertTextItem->Enabled = true;
      PopInsertFilenameItem->Enabled = true;
      PopWordWrapItem->Checked = Editor->WordWrap;
   }
   else if ((item=dynamic_cast<TMenuItem*>(Sender))!=NULL)
   {
      if (item->Caption=="&Undo") Editor->Perform(WM_UNDO,0,NULL); else
      if (item->Caption=="Cu&t")  CutToClipboard(); else
      if (item->Caption=="&Copy") Editor->CopyToClipboard(); else
      if (item->Caption=="Select &All") Editor->SelectAll(); else
      if (item->Caption=="&Paste") PasteFromClipboard(); else
      if (item->Caption=="Paste as &Quotation") PasteFromClipboardQuote(); else
      if (item->Caption=="Insert &Text From File...") InsertFileText(); else
      if (item->Caption=="Insert &Filename...") InsertFileName(); else
      if (item->Caption=="&Word Wrap This Message") ToggleWordWrap(); else
      if (item->Caption=="&Word Wrap This Note") ToggleWordWrap();
   }
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::ToggleWordWrap()
{
   SetRedraw(Editor,false);
   Editor->WordWrap = !Editor->WordWrap;
   AppProcessMessages();
   SetRedraw(Editor,true);
}
//---------------------------------------------------------------------------
bool __fastcall TTextWin::InsertFileText()
{
   if (MainForm->OpenDialog->Execute())
   {
      TStringList *ins = new TStringList();
      ins->LoadFromFile(MainForm->OpenDialog->FileName);
      Editor->SelText = ins->Text;
      SetEditRect();
      delete ins;
      return true;
   }
   return false;
}
//---------------------------------------------------------------------------
bool __fastcall TTextWin::InsertFileName()
{
   if (MainForm->OpenDialog->Execute())
   {
      Editor->SelText = MainForm->OpenDialog->FileName;
      SetEditRect();
      return true;
   }
   return false;
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::ShowCurXY(bool DontClear)
{
   if (!DontClear)
      Application->MainForm->Perform(WM_DISPLAYXY,-1,-1);
   else
   {
      long sel=Editor->SelStart;
      int linenum = Editor->Perform(EM_EXLINEFROMCHAR,0,(LPARAM)(DWORD)sel);
      int linepos = Editor->Perform(EM_LINEINDEX,linenum,0L);
      Application->MainForm->Perform(WM_DISPLAYXY,sel-linepos+1,linenum+1);
   }
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (Editor->Modified)
   {
      if ((ModalResult==mrCancel || ModalResult== mrNone) && !CheckSave())
      {
         CanClose = false;
         return;
      }
      if (ModalResult==mrOk)
         Save();
   }
   if (MDIChildCount==0) SetParentButtons(false);
   ShowCurXY(false);
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::WMMdiActivate(TMessage& Msg)
{
  // this function fixes the microsoft bug regarding
  // the 'disabled' close button in MDI window caption bar
    if (ActiveMDIChild && WindowState==wsMaximized)
    {
        Perform(WM_SETREDRAW, false, 0);
        try
        {
            ActiveMDIChild->Perform(CM_RECREATEWND, 0, 0);
        }
        catch (...)
        {
            Perform(WM_SETREDRAW, true, 0);
        }
        Perform(WM_SETREDRAW, true, 0);
        DrawMenuBar(Handle);
        ActiveMDIChild->Invalidate();
    }
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::FormActivate(TObject *Sender)
{
   SetParentButtons(true);
   ShowCurXY(true);
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::FormDeactivate(TObject *Sender)
{
   SetParentButtons(false);
   ShowCurXY(false);
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::EditorSelectionChange(TObject *Sender)
{
   SetParentButtons(true);
   ShowCurXY(true);
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::FormClose(TObject *Sender, TCloseAction &Action)
{
   Action = caFree;
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::OkBtnClick(TObject *Sender)
{
   ModalResult = mrOk;
   Close();
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::Print()
{
   if (MsgBoxHwnd(Handle,"Print window contents now?","Printer",MB_OKCANCEL)==IDOK)
      PrintMessage(Editor->Lines,Editor->Font,"Print Message");
}
//---------------------------------------------------------------------------
AnsiString __fastcall TTextWin::Name()
{
   return (Type==tNewNote ? newNote : IndexStr);
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::SetText(AnsiString &name, TStrings *Text)
{
   IndexStr = name;
   DataKey = UserNotes+PS+name;
   Caption = UserNotes+": "+name;
   Editor->OnSelectionChange = NULL;
   Editor->Lines->Text = Text->Text;
   Editor->Modified = false;
   Editor->SelStart = 0;
   Editor->OnSelectionChange = EditorSelectionChange;
   ShowCurXY(true);
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::SetParentButtons(bool enable)
{
   Application->MainForm->Perform(WM_SETBUTTONS,enable,Editor->SelLength);
   SetEditButtons(enable);
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::SetEditButtons(bool enable)
{
   long sel = Editor->SelLength;
   if (enable)
   {
      CutBtn->Enabled = sel;
      CopyBtn->Enabled = sel;
      PasteBtn->Enabled = Clipboard()->HasFormat(CF_TEXT);
   }
   else
   {
      PasteBtn->Enabled = false;
      CopyBtn->Enabled = false;
      CutBtn->Enabled = false;
   }
}
//----------------------------------------------------------------------------
void __fastcall TTextWin::SetEditRect()
{
   TRect Rct = Rect(4,0,Editor->ClientWidth-2,ClientHeight);
   Editor->Perform(EM_SETRECT,0,long(&Rct));
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::FormResize(TObject *Sender)
{
   SetEditRect();
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::FormPaint(TObject *Sender)
{
   SetEditRect();
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::EditorMouseMove(TObject *Sender, TShiftState Shift,
  int X, int Y)
{
   rnd_mouse(WM_MOUSEMOVE,0,MAKELONG(X,Y));
   rnd_mouse_wipe(X,Y);
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::PasteFromClipboardQuote()
{
   TMemo *m = new TMemo((void*)NULL);
   m->WordWrap = false;
   m->Visible = false;
   m->Parent = Application->MainForm;
   m->PasteFromClipboard();
   for (int i=0; i<m->Lines->Count; i++)
      m->Lines->Strings[i] = QteMarks+m->Lines->Strings[i];
   Editor->SelText = m->Lines->Text;
   SetEditRect();
   delete m;
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::PasteFromClipboard()
{
   TMemo *m = new TMemo((void*)NULL);
   m->WordWrap = false;
   m->Visible = false;
   m->Parent = Application->MainForm;
   m->PasteFromClipboard();
   Editor->SelText = m->Lines->Text;
   Editor->Modified = true;
   SetEditRect();
   delete m;
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::CutToClipboard()
{
   Editor->CutToClipboard();
   Editor->Modified = true;
   SetEditRect();
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::ClipboardBtnClick(TObject *Sender)
{
   TOfficeButton *btn = dynamic_cast<TOfficeButton*>(Sender);
   if (btn)
   {
      switch (btn->Tag)
      {
         case 3: PasteFromClipboard(); break;
         case 2: Editor->CopyToClipboard(); break;
         case 1: CutToClipboard(); break;
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TTextWin::GetUnWrappedText(TStrings *ToStrings)
{
   ::GetUnWrappedText(Editor,ToStrings); //func in global
}
//---------------------------------------------------------------------------
